package xyz.etztech.minecraftmanager.listeners;

import net.md_5.bungee.api.chat.BaseComponent;
import net.md_5.bungee.api.chat.ComponentBuilder;
import net.md_5.bungee.api.chat.HoverEvent;
import net.md_5.bungee.api.chat.TextComponent;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.player.AsyncPlayerChatEvent;
import xyz.etztech.minecraftmanager.*;

import java.io.FileWriter;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.regex.Pattern;

public class AsyncPlayerChatListener implements Listener {

    private MinecraftManager plugin;
    private List<String> staffPrefixes;

    public AsyncPlayerChatListener(MinecraftManager minecraftManager) {
        this.plugin = minecraftManager;
        reload();
        MCMUtil.init();
    }

    public void reload() {
        this.staffPrefixes = new ArrayList<>();
        this.staffPrefixes.addAll(trimmed(this.plugin.getConfig().getStringList("staff-chat.prefix")));
    }

    @EventHandler(priority=EventPriority.HIGHEST, ignoreCancelled=true)
    public void onChat(AsyncPlayerChatEvent event) {

        Player sender = event.getPlayer();
        String chat = event.getMessage();
        boolean staffChat = isStaffChat(sender, chat);
        boolean applyChat = MinecraftManager.inApplyMode(sender.getUniqueId().toString());

        TextComponent pre = new TextComponent(ChatColor.GOLD + sender.getName() + " > ");
        if (!applyChat) {
            if (staffChat) {
                event.setCancelled(true);
                for (String prefix : this.staffPrefixes) {
                    chat = chat.replaceFirst(Pattern.quote(prefix), "");
                }
                HoverEvent hover = new HoverEvent(HoverEvent.Action.SHOW_TEXT, new ComponentBuilder("Staff Chat").create());
                pre.setHoverEvent(hover);
                TextComponent message = generateMessage(chat);
                message.setHoverEvent(hover);
                for (Player player : event.getRecipients()) {
                    if (player.hasPermission("minecraftmanager.staff")) {
                        player.spigot().sendMessage(pre, message);
                    }
                }
                MCMUtil.log(MCMUtil.staffLog, componentText(pre, message));
            } else {
                TextComponent message = generateMessage(chat);
                MCMUtil.log(MCMUtil.globalLog, componentText(pre, message));
            }
        } else {
            // Applying
            event.setCancelled(true);

            Question applyQuestion = MinecraftManager.getApplyQuestion(sender.getUniqueId().toString());
            if ("stop".equalsIgnoreCase(chat)) {
                // Pause application
                MinecraftManager.setApplyMode(sender.getUniqueId().toString(), false);
                sender.sendMessage(ChatColor.GREEN + "Exiting apply mode. Use \"/apply\" to return to your application.");
            } else if ("last".equalsIgnoreCase(chat)) {
                // Previous question
                if (Question.ONE == applyQuestion) {
                    sender.sendMessage(ChatColor.RED + "This is the first question you ding dong. Say \"stop\" in chat to pause your application.");
                } else {
                    Question lastQuestion = Question.last(applyQuestion);
                    MinecraftManager.setApplyQuestion(sender.getUniqueId().toString(), lastQuestion);
                    sender.sendMessage(ChatColor.GREEN + lastQuestion.getQuestion());
                }
            } else {
                // Answering question
                boolean valid = Question.validate(applyQuestion, chat);
                boolean rules = this.plugin.getConfig().getBoolean("rules.application.validate");
                if (Question.FIVE == applyQuestion && !rules) {
                    valid = true;
                }
                if (!valid) {
                    sender.sendMessage(ChatColor.RED + applyQuestion.getError());
                    return;
                }
                Application app = MinecraftManager.getApplication(sender.getUniqueId().toString());
                app.setUsername(sender.getName());
                app.setAnswer(applyQuestion, chat);
                MinecraftManager.setApplication(sender.getUniqueId().toString(), app);
                Question nextQuestion = Question.next(applyQuestion, chat);
                MinecraftManager.setApplyQuestion(sender.getUniqueId().toString(), nextQuestion);

                if (Question.COMPLETE == nextQuestion) {
                    MinecraftManager.setApplyMode(sender.getUniqueId().toString(), false);
                    Bukkit.getConsoleSender().sendMessage(app.getFormatted());
                    MCMResponse response = new MCMResponse(MCMAPI.postApplication(app));
                    if (response.getStatus()) {
                        sender.sendMessage(ChatColor.GOLD + nextQuestion.getQuestion());
                        MCMUtil.staffMessage(ChatColor.GREEN + "New application #" + response.getExtra().getAsString() + " from " + sender.getName(), "/app info " + response.getExtra().getAsString());
                    } else {
                        sender.sendMessage(ChatColor.RED + response.getMessage());
                    }
                } else {
                    sender.sendMessage(ChatColor.GREEN + nextQuestion.getQuestion());
                }
            }
        }
    }

    private boolean isStaffChat(Player player, String message) {
        if (!player.hasPermission("minecraftmanager.staff")) {
            return false;
        }
        for (String prefix : this.staffPrefixes) {
            if (message.startsWith(prefix) && !message.startsWith(prefix+prefix)) {
                return true;
            }
        }
        return false;
    }

    private TextComponent generateMessage(String message) {
        TextComponent text = new TextComponent();
        for (String part : message.split(" ")) {
            text.addExtra(new TextComponent(ChatColor.GREEN + part + " "));
        }
        return text;
    }

    private List<String> trimmed(List<String> list) {
        for (int i = 0; i < list.size(); i++) {
            list.set(i, list.get(i).trim());
        }
        return list;
    }

    public String componentText(BaseComponent... components) {
        StringBuilder text = new StringBuilder();
        for (BaseComponent component : components) {
            text.append(ChatColor.stripColor(component.toPlainText()));
        }
        return text.toString();
    }

}
