package xyz.etztech.minecraftmanager.test;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import org.apache.commons.lang.StringUtils;
import xyz.etztech.minecraftmanager.Application;
import xyz.etztech.minecraftmanager.MCMAPI;
import xyz.etztech.minecraftmanager.MCMResponse;

import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;

public class Test {

    public static void main(String[] args) {
        String testUrl = "http://127.0.0.1:8000/api/";
        String testApi = "Testing1";
        try {
            testUrl = args[0];
            testApi = args[1];
        } catch (Exception ex) {
            System.out.println("Using default test URL and password");
        }
        System.out.println("URL: " + testUrl);
        System.out.println("Password: " + testApi);
        MCMAPI.test(testUrl, testApi);
        int option = 0;

        while (option != 99) {
            System.out.println("===== Test Util =====");
            System.out.println("1. Test Model Query - Search for applications with a username containing the number 1.");
            System.out.println("2. Test Application Posting - Post two applications");
            System.out.println("3. Test Application Action - Accept Testing1 application");
            System.out.println("4. Test Application Action - Deny Testing2 application");
            System.out.println("5. Test Application Clear - Clear Testing2 application");
            System.out.println("6. Test Login - Spoof a login of user Etzelia with IP 127.0.0.1");
            System.out.println("7. Test Ticket - Send a test ticket");
            System.out.println("99. Exit");
            System.out.print("Select Option: ");
            Scanner scanner = new Scanner(System.in);
            String input = scanner.next();
            int choice = StringUtils.isNumeric(input) ? Integer.parseInt(input) : 0;
            switch (choice) {
                case 1:
                    option = 1;
                    break;
                case 2:
                    option = 2;
                    break;
                case 3:
                    option = 3;
                    break;
                case 4:
                    option = 4;
                    break;
                case 5:
                    option = 5;
                    break;
                case 6:
                    option = 6;
                    break;
                case 7:
                    option = 7;
                    break;
                case 99:
                    option = 99;
                    break;
                default:
                    break;
            }


            if (option == 1) {
                // Query Model
                Map<String, String> filters = new HashMap<>();
                filters.put("username__icontains", "1");
                JsonArray results = MCMAPI.queryModel("application", filters);
                System.out.println(results.size());
                for (JsonElement jsonElement : results) {
                    System.out.println("Raw JSON: " + jsonElement.toString());
                    System.out.println("Username: " + jsonElement.getAsJsonObject().get("username").getAsString());
                    System.out.println("Ever Banned: " + jsonElement.getAsJsonObject().get("ever_banned").getAsBoolean());
                }
            } else if (option == 2) {
                // Post two applications
                Application app1 = new Application();
                app1.setUsername("Testing1");
                app1.setAge("20");
                app1.setPlayerType("First test application.");
                app1.setEverBanned("no");
                app1.setEverBannedExplanation("");
                app1.setReference("reddit");
                app1.setReadRules("24karrot");
                MCMResponse response1 = new MCMResponse(MCMAPI.postApplication(app1));
                System.out.println("===== App 1 =====");
                System.out.println("Status: " + response1.getStatus());
                System.out.println("Message: " + response1.getMessage());

                Application app2 = new Application();
                app2.setUsername("Testing2");
                app2.setAge("20");
                app2.setPlayerType("Second test application.");
                app2.setEverBanned("yes");
                app2.setEverBannedExplanation("Griefing");
                app2.setReference("planet minecraft");
                app2.setReadRules("24karrot");
                MCMResponse response2 = new MCMResponse(MCMAPI.postApplication(app2));
                System.out.println("===== App 2 =====");
                System.out.println("Status: " + response2.getStatus());
                System.out.println("Message: " + response2.getMessage());
            } else if (option == 3 || option == 4) {
                boolean accept = option == 3;
                String username = option == 3 ? "Testing1" : "Testing2";
                Map<String, String> filters = new HashMap<>();
                filters.put("username__exact", username);
                JsonArray array = MCMAPI.queryModel("application", filters);
                if (array.size() != 1) {
                    System.out.println("Couldn't find the application for " + username + ". Does it exist?");
                } else {
                    String id = array.get(0).getAsJsonObject().get("id").getAsString();
                    MCMResponse response = new MCMResponse(MCMAPI.postApplicationAction(id, accept, "Plugin Test"));
                    System.out.println("===== " + username + " =====");
                    System.out.println("Status: " + response.getStatus());
                    System.out.println("Message: " + response.getMessage());
                }
            } else if (option == 5) {
                Map<String, String> filters = new HashMap<>();
                filters.put("username__exact", "Testing2");
                JsonArray array = MCMAPI.queryModel("application", filters);
                if (array.size() != 1) {
                    System.out.println("Couldn't find the application for Testing2. Does it exist?");
                } else {
                    String id = array.get(0).getAsJsonObject().get("id").getAsString();
                    MCMResponse response = new MCMResponse(MCMAPI.postApplicationClear(id));
                    System.out.println("===== Testing2 =====");
                    System.out.println("Status: " + response.getStatus());
                    System.out.println("Message: " + response.getMessage());
                }
            } else if (option == 6) {
                MCMResponse response = new MCMResponse(MCMAPI.postLogin("Etzelia", "bf0446a8-9695-4c41-aa4c-7ff45bfd1171", "127.0.0.1"));
                System.out.println("===== Login =====");
                System.out.println("Status: " + response.getStatus());
                System.out.println("Message: " + response.getMessage());
            } else if (option == 7) {
                MCMResponse response = new MCMResponse(MCMAPI.postTicket("bf0446a8-9695-4c41-aa4c-7ff45bfd1171", "Test ticket.", "1", "2", "3", "O"));
                System.out.println("===== Ticket =====");
                System.out.println("Status: " + response.getStatus());
                System.out.println("Message: " + response.getMessage());
            }
        }
    }

}